/*
 * Plgui.cpp:
 *
 * This file is a port of my playlist generator, and is responsible for
 * manipulating playlists.
 *
 * Copyright (c) Malcolm Smith 2001-2005.  No warranty is provided.
 */

#include "pch.h"


int AddListItem(LIST * List,
				LPSTR szItem)
{
	if (List->dwItemsInList==List->dwItemsAllocated) {
		List->dwItemsAllocated+=GROWTH_INCREMENT;
		GlobalUnlock(List->hAlloc);
        List->hAlloc=GlobalReAlloc(List->hAlloc, (size_t)(List->dwItemsAllocated)*sizeof(LISTENTRY), GMEM_MOVEABLE);
		List->entrylist=(LISTENTRY*)GlobalLock(List->hAlloc);
		if (List->hAlloc==NULL) MessageBox(NULL, "Eeep, allocation error","",0);	
	}
	List->entrylist[List->dwItemsInList].hAlloc=GlobalAlloc(GMEM_MOVEABLE, strlen(szItem)+1);
	List->entrylist[List->dwItemsInList].data=(LPSTR)GlobalLock(List->entrylist[List->dwItemsInList].hAlloc);
	strcpy(List->entrylist[List->dwItemsInList].data,szItem);
	List->dwItemsInList++;
	return(0);
}

int SwapListItems(LIST * List,
                  DWORD dwItem1,
                  DWORD dwItem2)
{
    LISTENTRY temp;
    temp=List->entrylist[dwItem1];
    List->entrylist[dwItem1]=List->entrylist[dwItem2];
    List->entrylist[dwItem2]=temp;
    return 0;
}

int DeleteListItem(LIST * List,
                   DWORD dwItem)
{
    if (dwItem>=List->dwItemsInList) return 0;
    GlobalUnlock(List->entrylist[dwItem].hAlloc);
    GlobalFree(List->entrylist[dwItem].hAlloc);
    List->dwItemsInList--;
    if (List->dwItemsInList>0) {
        memmove(&List->entrylist[dwItem],
	        &List->entrylist[dwItem+1],
		sizeof(LISTENTRY)*(List->dwItemsInList-dwItem));
    }
    return 0;
}


int GenerateTree(LIST * List,
				 LPSTR szPath)
{
	char szTemp[MAX_PATH];
#ifndef WIN3X_BASEAPI
	char szTemp2[MAX_PATH];
#endif
	HANDLE FindHandle;
	pi_find FindData;
	sprintf(szTemp,"%s\\*.*",szPath);
	FindHandle=pi_findfirst(szTemp,&FindData);
	if (FindHandle!=INVALID_FIND_VALUE) {
		do {
			sprintf(szTemp,"%s\\%s",szPath,getName(&FindData));
			if (getAttr(&FindData)&FA_SUBDIR) {
				if (getName(&FindData)[0]!='.')
					GenerateTree(List,szTemp);
			} else {  
#ifndef WIN3X_BASEAPI
				GetFullPathName(szTemp,sizeof(szTemp2),szTemp2,NULL);
				InputFile(List,szTemp2);
#else
				InputFile(List,szTemp);
#endif
			}
		} while (pi_findnext(FindHandle,&FindData)!=INVALID_FIND_VALUE);
		pi_findclose(FindHandle);
	}
	return(0);
}

int ShuffleList(LIST * List)
{
	DWORD i,j;
	if (!List->dwItemsInList) return(0);
	for (i=0;i<List->dwItemsInList;i++) {
		j=(DWORD)List->dwItemsInList*rand()/RAND_MAX;
//		k=(int)ItemsInList*rand()/RAND_MAX;
		if (j!=i) {
			SwapListItems(List,i,j);
		}
        //if (canary1!=0x1e || canary2!=0x1e) DebugBreak();
	}
	return(0);
}

struct SearchData {
	LPSTR szFileName;
	DWORD dwItem;
};


int SortList(LIST * List,int iCriteria)
{
	DWORD BestItem;
	DWORD i,j;
	LPSTR a;
	DWORD li;
	struct SearchData * lpSD;
	LIST NewList;
	HANDLE hSD;

	struct SearchData sdTmp;
	
	li=List->dwItemsInList;

	if (li<2) return(0);

	hSD=GlobalAlloc(GMEM_MOVEABLE, li*sizeof(struct SearchData));
	lpSD=(struct SearchData *)GlobalLock(hSD);
	if (!lpSD) return(0);
	for (i=0;i<li;i++) {
		a=GetListItem(List, i);
		a=GetPtr(a,iCriteria);
		lpSD[i].szFileName=a;
		lpSD[i].dwItem=i;
	}

	for (i=0;i<(li-1);i++) {
		BestItem=i;
		for (j=(i+1);j<li;j++) 
			if (stricmp(lpSD[j].szFileName,lpSD[BestItem].szFileName)<0)
				BestItem=j;
		
		if (BestItem!=i) {
			memcpy(&sdTmp,&lpSD[BestItem],sizeof(struct SearchData));
			memcpy(&lpSD[BestItem],&lpSD[i],sizeof(struct SearchData));
			memcpy(&lpSD[i],&sdTmp,sizeof(struct SearchData));
		}
	}	

	NewList.hAlloc = GlobalAlloc(GMEM_MOVEABLE, sizeof(LISTENTRY)*li);
	if (NewList.hAlloc == NULL) {
		free(lpSD);
		return 0;
	}
    NewList.entrylist = (LISTENTRY*)GlobalLock(NewList.hAlloc);	

	for (i=0;i<li;i++) {
		NewList.entrylist[i].hAlloc = List->entrylist[lpSD[i].dwItem].hAlloc;
		NewList.entrylist[i].data = List->entrylist[lpSD[i].dwItem].data;
	}

    GlobalUnlock(List->hAlloc);
    GlobalFree(List->hAlloc);
	List->hAlloc=NewList.hAlloc;
    List->entrylist=NewList.entrylist;
	List->dwItemsAllocated=li;
	List->dwItemsInList=li;
	GlobalUnlock(hSD);
	GlobalFree(hSD);
	return(0);
}



int WriteList(LIST * List,LPSTR szFile,DWORD dwItems)
{
	char szTemp[MAX_PATH];
	char szMode[3];
	FILE * fp=NULL;
	DWORD i;
	DWORD seg=0;
	strcpy(szMode,"w");

	if (!dwItems) {
		fp=fopen(szFile,szMode);
	} else {
		LPSTR a;
		a=strrchr(szFile,'.');
		if (a) a[0]='\0';
	}
	for (i=0;i<List->dwItemsInList;i++) {
		if ((dwItems)&&(i%dwItems==0)) {
			if (fp) fclose(fp);
			fp=NULL;
			seg++;
			sprintf(szTemp,"%s%u.m3u",szFile,seg);
			fp=fopen(szTemp,szMode);
		}
		fprintf(fp,"%s\n",GetListItem(List,i));
	}
	if (fp) fclose(fp);
	return(0);
}

int InitList(LIST * List)
{
	List->hAlloc=GlobalAlloc(GMEM_MOVEABLE,GROWTH_INCREMENT*sizeof(LISTENTRY));
    List->entrylist=(LISTENTRY*)GlobalLock(List->hAlloc);
	List->dwItemsAllocated=GROWTH_INCREMENT;
	List->dwItemsInList=0;
	return(0);
}

int FreeList(LIST * List)
{
    GlobalUnlock(List->hAlloc);
    GlobalFree(List->hAlloc);
    List->hAlloc=NULL;
    List->entrylist=NULL;
	List->dwItemsInList=0;
	List->dwItemsAllocated=0;
	return(0);
}


int InputFile(LIST * List,
				 LPSTR szFile)
{
	char szTemp[MAX_PATH];
	char szExt[MAX_PATH];
	LPSTR a;
	FILE * fp;
	if (!strlen(szFile)) return(FALSE);
	//MessageBox(NULL,szFile,product,64);
	a=strrchr(szFile,'.');
	if (a) {
		strcpy(szExt,a+1);
		strupr(szExt);
#ifdef SCRIPTING
		if (!strcmp(szExt,"MSC")) {
			PlayScript(List,szFile);
			return(TRUE);
		}
#endif
		if (!strcmp(szExt,"M3U")) {
			fp=fopen(szFile,"r");
			if (fp) {
				while (fscanf(fp,"%[^\n]%*c",szTemp)>0) {
					if (strlen(szTemp))
						AddListItem(List,szTemp);
				}
				fclose(fp);
			}
			return(TRUE);
		}
		AddListItem(List,szFile);
	}
	return(TRUE);
}

int ClearList(LIST * List)
{
	CloseStream(GethWndMain());
	List->dwItemsInList=0;
	return(0);
}

LPSTR GetPtr(LPSTR szPath,DWORD dwTo)
{
	LPSTR a,b;
	a=szPath;

	if ((dwTo==SORT_FILE)||(dwTo==SORT_DISPLAY)) {
		if (b=strrchr(a,'\\')) a=b+1;
		if (dwTo==SORT_DISPLAY) {
			if (a[0]=='[') 
				if (b=strchr(a,']')) a=b+1;
		}
	}

	return(a);
}

int FindNext(LIST * List,DWORD dwStart,LPSTR szText,DWORD dwFlags)
{
	int dwFound=-1;
	char szTemp[MAX_PATH];
	LPSTR a;
	DWORD i;
	DWORD dwPtr;
	dwPtr=(dwFlags&FIND_SEARCHMASK)/FIND_SEARCHDIV;
	if (dwFlags&FIND_STARTTOP) dwStart=0;
	if (!(dwFlags&FIND_MATCHCASE)) {
		strlwr(szText);
	}
	if (dwStart>=List->dwItemsInList) return(-1);
	for (i=dwStart;((dwFound<0)&&(i<List->dwItemsInList));i++) {
		a=GetListItem(List,i);
		a=GetPtr(a,dwPtr);
		
		if (!(dwFlags&FIND_MATCHCASE)) {
			strcpy(szTemp,a);
			a=szTemp;
			strlwr(a);
		}
		if (!(dwFlags&FIND_INVERT)) {
			if (strstr(a,szText)) dwFound=(int)i;
		} else {
			if (!strstr(a,szText)) dwFound=(int)i;
		}
	}
	if (dwFound>=0) {
		return(dwFound);
	} else {
		return(-1);
	}
}
